#include "object.h"
#include "realbody.h"
#include "game.h"
#include <iostream>
#include <iomanip>
using namespace std;
namespace AntiKill
{

static int objectIdGenerator = 1;

Object::Object()
{
	bitmap = 0;
	width = height = 0;
	visible = false;
	dead = false;
	lifetime = -1;
	collidable = false;
	maxSpeed = 8.0;
	frictionConstant = 1.0;
	id = objectIdGenerator++;
	stop();
}

Object::~Object()
{
	if( bitmap )
		destroy_bitmap( bitmap );
	bitmap = 0;
}

}

void AntiKill::Object::draw(BITMAP* dest, int x, int y)
{
	int ox = getX();
	int oy = getY();
//	if( ox + width< x || oy+height<y || ox>=x+SCREEN_W || oy>=y+SCREEN_H )
//		return;
	if(bitmap){
		draw_sprite(dest, bitmap, ox-x, oy-y);
	}
}

Vector AntiKill::Object::getAcceleration() const
{
	return this->acceleration;
}

Vector AntiKill::Object::getPosition() const
{
	return this->position;
}

Vector AntiKill::Object::getVelocity() const
{
	return this->velocity;
}

int AntiKill::Object::getX() const
{
	return (int)(this->position.x+0.5);
}

int AntiKill::Object::getY() const
{
	return (int)(this->position.y+0.5);
}

void AntiKill::Object::move()
{
	Game& g = Game::getInstance();
	this->position += this->velocity;
	this->velocity += this->acceleration;
	float magnitude = this->velocity.getMagnitude();
	if( magnitude > this->maxSpeed ){
		this->velocity *= (this->maxSpeed/magnitude);
	}
}

AntiKill::Object& AntiKill::Object::setAcceleration(Vector& a)
{
	this->velocity = a;
	return *this;
}

AntiKill::Object& AntiKill::Object::setPosition(Vector& p)
{
	this->position = p;
	return *this;
}

AntiKill::Object& AntiKill::Object::setVelocity(Vector& v)
{
	this->velocity = v;
	return *this;
}

AntiKill::Object& AntiKill::Object::setX(int x)
{
	this->position.x = (float)x;
	return *this;
}

AntiKill::Object& AntiKill::Object::setY(int y)
{
	this->position.y = (float)y;
	return *this;
}

void AntiKill::Object::stop()
{
	Vector v(0.0,0.0);
	this->acceleration = v;
	this->velocity = v;
}

void AntiKill::Object::die()
{
	this->dead = true;
}

int AntiKill::Object::getHeight() const
{
	return this->height;
}

int AntiKill::Object::getLifetime() const
{
	return this->lifetime;
}

int AntiKill::Object::getWidth() const
{
	return this->width;
}

void AntiKill::Object::hide()
{
	this->visible = false;
}

bool AntiKill::Object::isDead() const
{
	return this->dead;
}

bool AntiKill::Object::isVisible() const
{
	return this->visible;
}

AntiKill::Object& AntiKill::Object::setHeight(int h)
{
	this->height = h;
	return *this;
}

AntiKill::Object& AntiKill::Object::setLifetime(int t)
{
	this->lifetime = t;
	return *this;
}

AntiKill::Object& AntiKill::Object::setWidth(int w)
{
	this->width = w;
	return *this;
}

void AntiKill::Object::show()
{
	this->visible = true;
}


bool AntiKill::Object::loadImage(const char* file)
{
	this->bitmap = load_bitmap(file, 0);
	if( !this->bitmap ){
		allegro_message("Failed to load bitmap %s", file);
		return false;
	}
	this->width = this->bitmap->w;
	this->height = this->bitmap->h;
	return true;
}

bool AntiKill::Object::collide(Object* o)
{
	RealBody* r = dynamic_cast<RealBody*>(this);
	const Rect& a = this->getRectangle();
	const Rect& b = o->getRectangle();
	if (b.x1> a.x2 || b.y1 > a.y2 || b.x2 < a.x1 || b.y2 < a.y1){
		if(r && r->getBoard()==o){
			r->setFree(true);
			r->setBoard(0);
		}
		return false;
	}
	if(r && r->getBoard() == o)
		return true;
	//Handle intersection problem!
	int dy1 = abs(a.y1-b.y2), dy2=abs(a.y2-b.y1);
	int dx1 = abs(a.x1-b.x2), dx2=abs(a.x2-b.x1);
	int dh = min(dy1, dy2), dw = min(dx1, dx2);
//	cout << "adjust dh: " << dh << " dw: " << dw << endl;
	if(dh<=1||dw<=1)
		return false;
	if(dh < dw){ //adjust y
		if(dy1<dy2){ //collid the bottom of o
			this->position.y = b.y2 + 1;
			this->handleCollision(DIR_TOP, o);
			o->handleCollision(DIR_BOTTOM, this);
		}else{ //collid the top of o
			this->position.y = b.y1 - a.y2 + a.y1 + 1;
			this->handleCollision(DIR_BOTTOM, o);
			o->handleCollision(DIR_TOP, this);
		}
	}else { //adjust x
		if(dx1<dx2){ //collid the right of o
			this->position.x = b.x2+1;
			this->handleCollision(DIR_LEFT, o);
			o->handleCollision(DIR_RIGHT, this);
		}else{ //collid the left of o
			this->position.x = b.x1 - a.x2 + a.x1 -1;
			this->handleCollision(DIR_RIGHT, o);
			o->handleCollision(DIR_LEFT, this);
		}
	}
	return true;
}

bool AntiKill::Object::isCollidable() const
{
	return this->collidable;
}

AntiKill::Object& AntiKill::Object::setCollidable(bool c)
{
	this->collidable = c;
	return *this;
}

const Rect& AntiKill::Object::getRectangle() 
{
	updateRectangle();
	return rect;
}

void AntiKill::Object::updateRectangle()
{
	rect.x1 = getX();
	rect.x2 = rect.x1 + this->width;
	rect.y1 = getY();
	rect.y2 = rect.y1 + this->height;
}

int AntiKill::Object::getCentralX() const
{
	return (getX()+getWidth()/2);
}

int AntiKill::Object::getCentralY() const
{
	return (getY()+getHeight()/2);
}

void AntiKill::Object::handleCollision(int dir, Object* o)
{
}

float AntiKill::Object::getFrictionConstant() const
{
	return this->frictionConstant;
}

MapLayer AntiKill::Object::getLayer() const
{
	return this->layer;
}

AntiKill::Object& AntiKill::Object::setLayer(MapLayer l)
{
	this->layer = l;
	return *this;
}

int AntiKill::Object::getId() const
{
	return this->id;
}

