#include "player.h"
#include "game.h"
#include "bullet.h"
#include "blood.h"
#include <iostream>

namespace AntiKill
{

Player::Player()
{
	initialize();
}

Player::~Player()
{
}

}

void AntiKill::Player::act(enum PlayerState st)
{
	switch(st){
	case NormalState:
		acting = false;
		this->animation.setAction(0);
		break;
	case WalkingState:
		acting = true;
		this->animation.setAction(0, true);
		break;
	case RunningState:
		break;
	case JumpingState:
		break;
	case ShootingState:
		break;
	case SquatingState:
		break;
	case DyingState:
		acting = true;
		this->animation.setAction(3, false);
		break;
	case DeadState:
		break;
	}
	this->state = st;
}

void AntiKill::Player::fire(int x, int y)
{
	firing = true;
	fireX = x;
	fireY = y;
}

int AntiKill::Player::getHealth() const
{
	return health;
}

string AntiKill::Player::getName() const
{
	return playerName;
}

bool AntiKill::Player::hit(int x, int y, Vector& dir)
{
	Game &g = Game::getInstance();
	int h = y - getY();
	int bloodCount = (int)( ( 1-((float) h / getHeight() ) ) * 10 );
	for(int i=0; i<bloodCount; i++){
		Blood *b = new Blood(x, y, dir);
		g.getObjectSystem().addObject(b);
	}
	if(isAlive()){
		setHealth(this->health - bloodCount);
		if(this->health <= 0 ){
			//raise event
			g.getScriptSystem().raise("OnKill", this->getId(), x, y);
			die();
		}
	}
	//stop firing if hit ?
	stopMoving();
	//head shot?
	if(bloodCount==10)
		return true;
	else
		return false;
}

void AntiKill::Player::jump(bool t)
{
	jumping = t;
}

void AntiKill::Player::moveLeft(bool t)
{
	if(!isAlive())
		return;
	movingLeft = t;
	if(movingLeft){
		this->animation.setDirection(DIR_LEFT);
		this->act(WalkingState);
	}else{
		if(movingRight)
			moveRight(true);
		else
			stopMoving();
	}
}

void AntiKill::Player::moveRight(bool t)
{
	if(!isAlive())
		return;
	movingRight = t;
	if(movingRight){
		this->animation.setDirection(DIR_RIGHT);
		this->act(WalkingState);
	}else{
		if(movingLeft)
			moveLeft(true);
		else
			stopMoving();
	}
}

void AntiKill::Player::setName(string name)
{
	this->playerName = name;
}

string AntiKill::Player::getRole() const
{
	return role;
}

void AntiKill::Player::setRole(string role)
{
	string file = "image/role/";
	this->role = role;
	file += role + "/config.dat";
	this->animation.load(file.c_str());
	this->act(NormalState);
	this->setWidth(this->animation.getFrameWidth());
	this->setHeight(this->animation.getFrameHeight());
}

void AntiKill::Player::draw(BITMAP* dest, int x, int y)
{
	int tx=this->getX()-x, ty=this->getY()-y;
	this->animation.draw(dest, tx, ty);
	if(isAlive()&&this->equipment&&this->weapons[this->equipment]){
		this->weapons[this->equipment]->draw(dest, this->getCentralX()-x, this->getCentralY()-y);
	}
}

void AntiKill::Player::move()
{
	//Animation!!!
	if(acting){
		int step;
		this->animation.nextFrame();
		this->setWidth(this->animation.getFrameWidth());
		this->setHeight(this->animation.getFrameHeight());
	}
	//Playing
	if(isAlive()){
		switch(state){
		case WalkingState:
			if(this->animation.getDirection()==DIR_LEFT){
				doMoveLeft();
			}else{
				doMoveRight();
			}
			break;
		}
		if(jumping && !this->isFree()){
			doJump();
		}
		if( firing ){
			doFire();
		}
		if(this->playerActionOn)
			doPlayerAction();
	}
	if(this->equipment && this->weapons[this->equipment]){
		this->weapons[this->equipment]->move();
	}
	//Object movement
	RealBody::move();
}


bool AntiKill::Player::isAlive() const
{
	return health!=0;
}

int AntiKill::Player::getPlayerId() const
{
	return this->playerId;
}

AntiKill::Player::Player(string name, string role)
{
	initialize();
	setName(name);
	setRole(role);
}

void AntiKill::Player::stopMoving()
{
	if(isAlive()){
		act(NormalState);
		movingLeft = movingRight = jumping = firing = false;
	}
}

void AntiKill::Player::initialize()
{
	playerName = "NoName";
	playerId = rand();
	health = 100;
	kills = 0;
	headshots = 0;
	acting = false;
	stepSize = 5;
	setMass(50);
	movingLeft = movingRight = jumping = firing = false;
	color = makecol(rand()%255, rand()%255, rand()%255);
	this->playerActionOn = false;
	equipment = 0;
	memset(weapons, 0, sizeof(weapons));
}

int AntiKill::Player::getColor() const
{
	return color;
}

void AntiKill::Player::holdFire()
{
	firing = false;
}

bool AntiKill::Player::isFiring() const
{
	return firing;
}

bool AntiKill::Player::collide(Object* o)
{
	bool collision;
	Blood* b = dynamic_cast<Blood*>(o);
	Player* p = dynamic_cast<Player*>(o);
	if(b || (p&&!p->isAlive()) ){
		collision = false;
	}else{
		collision = RealBody::collide(o);
	}
	//Stop action if we collide something
	if(collision && o!=this->getBoard() && this->playerActionOn)
		this->playerActionOn = false;
	return collision;
}

void AntiKill::Player::die()
{
	this->health = 0;
	act(DyingState);
}

void AntiKill::Player::setColor(int i)
{
	this->color = i;
}

void AntiKill::Player::addKills()
{
	this->kills ++;
}

int AntiKill::Player::getKills() const
{
	return this->kills;
}

void AntiKill::Player::addHeadshots()
{
	this->headshots ++;
}

int AntiKill::Player::getHeadshots() const
{
	return this->headshots;
}

void AntiKill::Player::setHealth(int h)
{
	this->health = h;
}

bool AntiKill::Player::isPlayerActionOn() const
{
	return this->playerActionOn;
}

void AntiKill::Player::setAction(PlayerAction& a)
{
	this->playerAction = a;
	this->playerActionOn = true;
}

void AntiKill::Player::stopAction()
{
	this->playerActionOn = false;
}

void AntiKill::Player::doFire()
{
	if(this->equipment&&this->weapons[this->equipment])
		this->weapons[this->equipment]->fire(this->fireX, this->fireY);
}

void AntiKill::Player::doJump()
{
	Vector v(0.0, -300.0);
	this->addMomentum(v);
}

void AntiKill::Player::doMoveLeft()
{
	if(!isFree()){
		Vector v(-10.0,0.0);
		this->addMomentum(v);
	}else{
		this->position.x --;
	}
}

void AntiKill::Player::doMoveRight()
{
	if(!isFree()){
		Vector v(10.0,0.0);
		this->addMomentum(v);
	}else{
		this->position.x ++;
	}
}

void AntiKill::Player::doPlayerAction()
{
	switch(playerAction.type){
	case ActionMove:{
		float x = playerAction.x;
		if(fabs(this->position.x-x) < 1.0 ){
			this->playerActionOn = false;
		}else{
			if(this->position.x < x){
				this->velocity.x = fmin(3.0, x-this->position.x);
				if(!acting){
					this->animation.setDirection(DIR_RIGHT);
					acting = true;
					this->animation.setAction(0, true);
				}
			}else{
				this->velocity.x = -fmin(3.0, this->position.x-x);
				if(!acting){
					this->animation.setDirection(DIR_LEFT);
					acting = true;
					this->animation.setAction(0, true);
				}
			}
		}
		break;
	}
	}
}


void AntiKill::Player::aim(int x, int y)
{
	if(isAlive()){
		aimX = x;
		aimY = y;
		if(this->equipment&&this->weapons[this->equipment]){
			this->weapons[this->equipment]->setTarget(x-this->getCentralX(), this->getCentralY()-y);
		}
	}
}

int AntiKill::Player::getDefend() const
{
	return this->defend;
}

void AntiKill::Player::setDefend(int d)
{
	this->defend = d;
}

AntiKill::Weapon* AntiKill::Player::getCurrentWeapon()
{
	return this->weapons[this->equipment];
}

int AntiKill::Player::getEquipment() const
{
	return this->equipment;
}

void AntiKill::Player::setEquipment(int v)
{
	this->equipment = v;
}

AntiKill::Weapon* AntiKill::Player::getWeapon(int e)
{
	return this->weapons[e];
}

void AntiKill::Player::setWeapon(int e, Weapon* w)
{
	if(this->weapons[e]){
		if(this->equipment==e)
			this->equipment = EquipNone;
		delete this->weapons[e];
		
	}
	this->weapons[e] = w;
}

void AntiKill::Player::setWeapon(string type)
{
}

